use crate::agents::audio_agent::{self, AudioAgent, AudioData};
use web_sys::HtmlCanvasElement;
use yew::agent::Bridged;
use yew::prelude::*;
use yew::Bridge;

pub struct Canvas {
    _audio_agent: Box<dyn Bridge<AudioAgent>>,
    canvas_node: NodeRef,
    audio_data: Option<AudioData>,
}

#[derive(Debug)]
pub enum Msg {
    Reset,
    HandleAudioAgentResponse(audio_agent::Response),
}

impl Component for Canvas {
    type Message = Msg;
    type Properties = ();

    fn create(_: Self::Properties, link: ComponentLink<Self>) -> Self {
        Self {
            _audio_agent: AudioAgent::bridge(link.callback(Msg::HandleAudioAgentResponse)),
            canvas_node: NodeRef::default(),
            audio_data: None,
        }
    }

    fn update(&mut self, msg: Self::Message) -> ShouldRender {
        match msg {
            Msg::Reset => {
                self.audio_data = None;
                self.redraw_canvas();
            }
            Msg::HandleAudioAgentResponse(audio_data) => match audio_data {
                audio_agent::Response::SamplesLoaded(audio_data) => {
                    self.handle_samples_loaded(audio_data)
                }
            },
        };
        false
    }

    fn change(&mut self, _: Self::Properties) -> ShouldRender {
        false
    }

    fn view(&self) -> Html {
        html! { <canvas ref=self.canvas_node.clone() width="800" height="300" style="border: 1px solid grey"></canvas> }
    }
}

impl Canvas {
    fn handle_samples_loaded(&mut self, audio_data: AudioData) {
        self.audio_data = Some(audio_data);
        self.redraw_canvas();
    }

    fn redraw_canvas(&mut self) {
        let canvas_element = self.canvas_node.cast::<HtmlCanvasElement>().unwrap();
        let _context = canvas_element.get_context("2d").unwrap();
        // TODO: draw canvas...
    }
}
